//= require <util>

Ext.namespace('dSS11', 'dSS11.config', 'dSS11.util');

dSS11.AddOnsFeedPanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.feedStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=addon-list',
                    root: 'data',
                    fields: 
                    [
                        'name', 'version', 'maintainer', 
                        'description', 'homepage', 'state'
                    ]
                }
            );

            this.loaded = false;
            this.refetched = false;

            var tpl = dSS11.util.defaultIPKTemplate();
            tpl.compile();

            this.mainGrid = new Ext.grid.GridPanel(
                {
                    store: this.feedStore,
                    enableColumnHide: false,
                    enableHdMenu: false,
                    columns:
                    [
                        {
                            resizable: false,
                            sortable: false,
                            hideable: false,
                            menuDisabled: true,
                            fixed: true,
                            renderer: dSS11.util.defaultAddOnIconRenderer,
                            width: 64
                        },
                        {
                            xtype: 'templatecolumn',
                            id: 'addon',
                            header: _("Apps"),
                            dataIndex: 'name',
                            sortable: true,
                            tpl: tpl
                        }
                    ],
                    trackMouseOver: true,
                    viewConfig: { forceFit: true },
                    sm: new Ext.grid.RowSelectionModel({singleSelect:true}),
                    bodyBorder: false
                }
            );

            this.refetchTip = new Ext.ToolTip(
                {
                    target: 'dSS11AddonsRefetchButton',
                    title: _("Update list of packages"),
                    html: '<br/>' +
                         _("Make sure to click this button to update the " + 
                           "list of available packages."),
                    closable: true,
                    draggable: true,
                    anchor: 'top'
                }
            );

            Ext.apply(this,
                {
                    title: _("Install"),
                    layout: 'fit',
                    tbar: [
                        {
                            xtype: 'button',
                            text: _("Refetch list from server"),
                            id: 'dSS11AddonsRefetchButton',
                            iconCls: 'icon-refresh-feed',
                            tooltip: _("Download latest package information " +
                                       "list from the server"),
                            scope: this,
                            handler: this.handleRefetch.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Install selected"),
                            tooltip: _("Download and install selected app"),
                            iconCls: 'icon-package-add',
                            id: 'installButton',
                            scope: this,
                            handler: this.installSelected.createDelegate(this),
                            disabled: true
                        },
                        '->',
                        {
                            xtype: 'button',
                            text: _("Disclaimer"),
                            tooltip: _("View disclaimer"),
                            iconCls: 'icon-disclaimer',
                            scope: this,
                            handler: this.showDisclaimer.createDelegate(this)
                        }
                    ],
                    
                    items: [ this.mainGrid ]
                }
            );

            dSS11.AddOnsFeedPanel.superclass.initComponent.apply(this, arguments);

            dSS11.MessageBus.on('refreshaddonsview', function() {
                    if (this.isVisible()) {
                        this.refreshList();
                    }
                },
                this);

            this.on('activate',
                function() {
                    if (this.loaded === false) {
                        this.feedStore.load();
                        this.loaded = true;
                    }
                },
                this);

            this.feedStore.on('load',
                function() 
                {
                    if ((this.feedStore.getTotalCount() === 0) && 
                        (this.refetched === false) && (this.isVisible())) {
                        this.refetchTip.show();
                    }
                },
                this);

            this.mainGrid.getSelectionModel().on('selectionchange',
                function(sel)
                {
                    if (sel.hasSelection()) {
                        this.getTopToolbar().get('installButton').enable();
                    } else {
                        this.getTopToolbar().get('installButton').disable();
                    }
                },
                this);

        },
        showDisclaimer: function()
        {
            dSS11.util.showDisclaimer(true,
                    this.findParentByType('viewport').el);
        },
        refreshList: function() 
        {
            this.feedStore.load();
        },
        refetchDone: function()
        {
            this.refetched = true;
            this.refetchTip.hide();
            this.refetchTip.disable();
            this.refreshList();
        },
        handleRefetch: function() 
        {
            var fn = dSS11.util.opkg.createDelegate(this, 
                           [ 'addon-update', 
                              this.findParentByType('viewport').el,
                              this.refetchDone.createDelegate(this) ]);
            fn();
        },
        installSelected: function()
        {
            var sel = this.mainGrid.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.MessageBox.alert(_("No app selected"),
                      _("Please select an app that should be installed!"));
                return;
            }

            var record = sel.getSelected();
            var name = record.get('name');

            var fn = dSS11.util.opkg.createDelegate(this,
                    [ 'addon-install',
                       this.findParentByType('viewport').el,
                       this.doneInstall.createDelegate(this),
                       name ]);
            fn();
        },
        doneInstall: function(jsonData, name) {
            var msg = _(jsonData.message);
            var title = _("Info");

            if (jsonData.code === dSS11.config.codes.ERR_OK) {
                msg = _("App") + " <b>" + name + "</b> " +
                      _("was successfully installed on your system");
            } else if (jsonData.code === dSS11.config.codes.ERR_OPKG) {
                if (!(jsonData.opkgcode &
                    ~(dSS11.config.opkgcodes.ERR_PACKAGE_ALREADY_INSTALLED |
                      dSS11.config.opkgcodes.ERR_PACKAGE_NOT_DOWNGRADED))) {
                    // these are errors that we can usually ignore
                    msg = _("App") + " <b>" + name + "</b> " +
                          _("was successfully installed on your system");
                } else if (jsonData.opkgcode ===
                    dSS11.config.opkgcodes.ERR_DOWNLOAD_FAILED) {
                    msg = _("Failed to download packages, please try again " +
                            "later!");
                    title = _("Error");
                } else {
                    msg = _("App") + " <b>" + name + "</b> " +
                          _("could not be installed.");
                    title = _("Error");
                }
            }

            dSS11.StatusBar.setMessage(msg);
            Ext.Msg.show({
                title: title,
                msg: msg,
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK
            });
            this.refreshList();
            this.findParentByType('dss11listmenupanel').getItem('dSS11AddonsOverview').refreshOverview();
        }
        
    }
);

Ext.reg('dss11addonsfeed', dSS11.AddOnsFeedPanel);
